#include <FdIOBuf.h>

FrameH *FdIOGetFrameFromBufDoIt(FdIO* fdIO, char *buf, FRLONG nBytes, int comp)
/*----------------------------------------------------------------------------*/
{
  FrameH *frame;
  int i, wait, latency;
  double frameGTime, delta;

  if(fdIO->actionsIn == NULL) 
    CfgMsgAddFatal("FdIOGetFrame called but not FDIN actions requested");

  frame = FrameReadFromBuf(buf,nBytes,comp);


  /*--------------no frame available; go througth the input list of actions---*/
  if(fdIO->iFrameListIndex < 0) 
    fdIO->actionsIn->action(fdIO->actionsIn->data, frame);

  /*--do we have a new frame? if yes, take the first one and shift the list---*/
  if(fdIO->iFrameListIndex >= 0) {
    frame = fdIO->iFrameList[0];
    for(i = 0; i< fdIO->iFrameListIndex; i++) {
      fdIO->iFrameList[i] = fdIO->iFrameList[i+1];}
    fdIO->iFrameListIndex--;}
  else {
    return(NULL);}

  /*--------------------------------------- reject frame with no GPS time ---*/
  if (frame->GTimeS == 0) {
    CfgMsgAddError("FdIOGetFrame: Reject frame %s without GPS",frame->name);
    FrameFree(frame);
    return(NULL);}

  /*--------------------------------------------------reject futur frames ---*/
  if((latency = FrameLatency(frame)) < fdIO->minLatency) {
    CfgMsgAddError("FdIOGetFrame: Reject futur frame: GPS=%d latency=%d",
		   frame->GTimeS, latency);
    FrameFree(frame);
    return(NULL);}

  /*---------------------------------------------------reject late frames ---*/
  frameGTime = frame->GTimeS + 1.e-9*frame->GTimeN;
  if (frameGTime <= fdIO->lastIFrameGTime && fdIO->lastIFrameGTime > 0) {
    CfgMsgAddError( "FdIOGetFrame: Reject late frame: GPS=%.1f (last=%.1f)",
		    frameGTime, fdIO->lastIFrameGTime);
    FrameFree(frame);
    return(NULL);}

  /*-----------------------------------------warning for missing frames ---*/
  delta =fabs(fdIO->lastIFrameGTime + fdIO->lastIFrameDt - frameGTime); 
  if ((delta > 0.001*fdIO->lastIFrameDt) && (fdIO->lastIFrameGTime > 0)) {
    CfgMsgAddWarning ( "FdIOGetFrame: miss %g seconds between %.1f and %.1f",
		       delta,      fdIO->lastIFrameGTime+fdIO->lastIFrameDt, 
		       frameGTime);}

  fdIO->lastIFrameGTime = frameGTime;
  fdIO->lastIFrameDt    = frame->dt;

  /*--------------------------------------------remove FrMsg if requested ---*/
  if(fdIO->deleteMsg == FR_YES && frame->rawData != NULL) {
    FrMsgFree(frame->rawData->logMsg);
    frame->rawData->logMsg = NULL;}

  /*------------------------------------ check reading speed if requested ---*/
  if (fdIO->maxSpeed > 0 && fdIO->nFramesGet > 0) {
    if(fdIO->tStart == 0) fdIO->tStart = time(NULL);
    wait =  fdIO->nFramesGet/fdIO->maxSpeed + fdIO->tStart - time(NULL);
    if(wait > 0) sleep(wait);}
      
  return(frame);}

/*----------------------------------------------------------------------------*/
FrameH *FdIOGetFrameFromBuf(FdIO* fdIO, char *buf, FRLONG nBytes,int comp)
/*----------------------------------------------------------------------------*/
{
  FrameH *frame;
  int dt, state;
  struct timeval tS, tE;

  gettimeofday(&tS, NULL);
  state = CfgServerGolden;

  frame = FdIOGetFrameFromBufDoIt(fdIO,buf,nBytes,comp);

  /*------------if no frame update the program state and messages if needed--*/
  if (frame == NULL) {
    dt = time(NULL)-fdIO->lastFrameTime;
    if (dt > fdIO->noFrMsgTimeOut + 2*fdIO->lastIFrameDt) {
      if (fdIO->getFrStatus == FR_YES)
	CfgMsgAddWarning("No input frame since at least %d seconds", dt);
      fdIO->getFrStatus = FR_NO;
      state = CfgServerActive;
      CfgMsgAddUserInfo("Warning: no input frame since %d s", dt);}}

  /*----------- a frame is available; update messages and last frame info ---*/
  else {
    if (fdIO->getFrStatus == FR_NO) 
      CfgMsgAddInfo("Input frames are back; gps=%d latency=%.1f",
                    frame->GTimeS, FrameLatency(frame));
    fdIO->getFrStatus = FR_YES;
    fdIO->lastFrameTime = time(NULL);
    fdIO->nFramesGet ++;

    /*-------------------------------------------- update input statistics ---*/
    gettimeofday(&tE, NULL);
    fdIO->inputTime = tE.tv_sec -tS.tv_sec + 1.e-6 * (tE.tv_usec - tS.tv_usec);}

  /*--------------------------------------------------update program state---*/
  FdUpdateState(fdIO, state);

  /*----------------------------------------------------flush Cfg messages---*/
  if(fdIO->getFrTimeOut >= 0) CfgMsgSendWithTimeout(fdIO->getFrTimeOut);

  return(frame);
}

